package com.muyu.kvt.service.impl;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;


import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.common.system.domain.SysDept;
import com.muyu.common.system.domain.SysUser;
import com.muyu.kvt.dataStructure.OverallDataStructure;
import com.muyu.kvt.datadisplay.DataDisplay;
import com.muyu.kvt.dept.DeptUserKvt;
import com.muyu.kvt.dictionary.Diction;
import com.muyu.kvt.dictionary.DictionaryType;
import com.muyu.kvt.dictionary.Dictionaryy;
import com.muyu.kvt.domain.*;

import com.muyu.kvt.remote.RemoteUserSer;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.kvt.mapper.KvtMapper;
import com.muyu.kvt.service.KvtService;


/**
 * kvt
 * Service业务层处理
 *
 * @author muyu
 * @date 2024-04-20
 */
@Slf4j
@Service
public class KvtServiceImpl extends ServiceImpl<KvtMapper, Kvt> implements KvtService {

    /**
     * 查询kvt列表
     *
     * @param kvt kvt
     * @return kvt
     */
    @Override
    public List<Kvt> list(Kvt kvt) {
        LambdaQueryWrapper<Kvt> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(kvt.getName())) {
            queryWrapper.like(Kvt::getName, kvt.getName());
        }

        if (ObjUtils.notNull(kvt.getSystemName())) {
            queryWrapper.like(Kvt::getSystemName, kvt.getSystemName());
        }

        if (ObjUtils.notNull(kvt.getType())) {
            queryWrapper.eq(Kvt::getType, kvt.getType());
        }

        if (ObjUtils.notNull(kvt.getHost())) {
            queryWrapper.eq(Kvt::getHost, kvt.getHost());
        }

        if (ObjUtils.notNull(kvt.getPort())) {
            queryWrapper.eq(Kvt::getPort, kvt.getPort());
        }

        if (ObjUtils.notNull(kvt.getDatabaseName())) {
            queryWrapper.like(Kvt::getDatabaseName, kvt.getDatabaseName());
        }

        if (ObjUtils.notNull(kvt.getConnectionParam())) {
            queryWrapper.eq(Kvt::getConnectionParam, kvt.getConnectionParam());
        }

        if (ObjUtils.notNull(kvt.getInitNum())) {
            queryWrapper.eq(Kvt::getInitNum, kvt.getInitNum());
        }

        if (ObjUtils.notNull(kvt.getMaxNum())) {
            queryWrapper.eq(Kvt::getMaxNum, kvt.getMaxNum());
        }

        if (ObjUtils.notNull(kvt.getMaxWaitTime())) {
            queryWrapper.eq(Kvt::getMaxWaitTime, kvt.getMaxWaitTime());
        }

        if (ObjUtils.notNull(kvt.getMaxWaitSize())) {
            queryWrapper.eq(Kvt::getMaxWaitSize, kvt.getMaxWaitSize());
        }


        return list(queryWrapper);
    }

    @Override
    public boolean connectionTest(Kvt kvt) throws ClassNotFoundException, SQLException {
        try {
            if (kvt.getType() == 1) {
                return Mysql(kvt);
            } else {
                return PostgreSQL(kvt);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    public boolean Mysql(Kvt kvt) throws SQLException, ClassNotFoundException {
        String user = kvt.getUsername();
        String password = kvt.getPassword();
        String jdbcDriver = "com.mysql.cj.jdbc.Driver";
        String jdbcUrl = "jdbc:mysql://" + kvt.getHost() + ":" + kvt.getPort() + "/" + kvt.getDatabaseName() + "?" + kvt.getConnectionParam();
        Class.forName(jdbcDriver);
        Connection connection = DriverManager.getConnection(jdbcUrl, user, password);
        if (connection == null) {
            return false;
        }
        return true;
    }

    public boolean PostgreSQL(Kvt kvt) throws SQLException, ClassNotFoundException {
        String user = kvt.getUsername();
        String password = kvt.getPassword();
        String jdbcDriver = "org.postgresql.Driver";
        String jdbcUrl = "jdbc:postgresql://" + kvt.getHost() + ":" + kvt.getPort() + "/" + kvt.getDatabaseName();
        Class.forName(jdbcDriver);
        Connection connection = DriverManager.getConnection(jdbcUrl, user, password);
        if (connection == null) {
            return false;
        }
        return true;
    }

    @Override
    public List<DataType> dataTypeList() {
        return baseMapper.dataTypeList();
    }


    @Override
    public List<AssetStructureList> assetStructureList() {
        List<AssetStructureList> assetStructureListList = baseMapper.assetStructureList();

        //   private Integer kvtNumber;
        //    private Integer tableSum;
        //    private Integer sum;
        int kvtNumber = assetStructureListList.size();
        int tableSum = 0;
        int sum = 0;
        int count = 0;

        return assetStructureListList;
    }

    @Override
    public List<ChildrenList> selectDepartment(String databaseName) {
        List<ChildrenList> list = baseMapper.selectDepartment(databaseName);
//          Kvt kvt =  baseMapper.selectType(databaseName);
        List<ChildrenList> childrenLists = new ArrayList<>();
        for (ChildrenList childrenList : list) {
            ChildrenList build = ChildrenList.builder()
                    .name(childrenList.getName())
                    .dataTotal(childrenList.getDataTotal())
                    .as(childrenList.getAs())
                    .type(Long.valueOf(2))
                    .build();
            childrenLists.add(build);
        }
        return childrenLists;

    }

    @Override
    public List<DataStructure> selectDataStructure(String databaseName, String name) {
        return baseMapper.selectDataStructure(databaseName, name);
    }

    @Override
    public void add(DataStructure dataStructure) {
        baseMapper.add(dataStructure);
    }

    @Override
    public List<DataStructure> selectDataStur(String tableName) {
        return baseMapper.selectDataStur(tableName);
    }

    @Override
    public Child selectChild(String tableName) {
        return baseMapper.selectChild(tableName);
    }

    @Override
    public List<DataStructure> selectChildAll() {
        return baseMapper.selectChildAll();
    }

    @Override
    public List<SysDept> deptList() {
        return baseMapper.deptList();
    }

    @Override
    public List<Diction> selectDictionaryy(Integer id) {

        List<DictionaryType> dictions = baseMapper.selectDictionaryy(id);

        List<Diction> dictions1 = dictions.stream()
                .map(dictionaryType -> {
                    List<Dictionaryy> list = baseMapper.selectDiction(dictionaryType.getType());
                    Diction build = Diction.builder()
                            .id(dictionaryType.getId())
                            .status(dictionaryType.getStatus())
                            .dictionaryTypes(list)
                            .name(dictionaryType.getName())
                            .type(dictionaryType.getType())
                            .build();
                    return build;
                }).toList();
        return dictions1;

    }

    @Override
    public List<DataAuthorization> selectDataAuthorization(String databaseName) {

        List<ChildrenList> list = baseMapper.selectDepartment(databaseName);
        return null;
    }

    @Autowired
    private RemoteUserSer remoteUserSer;

    public Result mysql(Kvt kvt) {

        //用户名
        String user = kvt.getUsername();
        //密码
        String password = kvt.getPassword();
        //驱动程序
        String jdbcDriver = "com.mysql.cj.jdbc.Driver";
        //连接对象
        String jdbcUrl = "jdbc:mysql://" + kvt.getHost() + ":" + kvt.getPort() + "/" + kvt.getDatabaseName() + "?" + kvt.getConnectionParam();
        Connection connection = null;
        String sql = " ";
        try {
            Class.forName(jdbcDriver);

            connection = DriverManager.getConnection(jdbcUrl, user, password);
             if (connection == null){
                 return Result.error("连接失败");
             }
            Statement statement = connection.createStatement();
            DatabaseMetaData metaData = connection.getMetaData();
            //添加到同步表中
            baseMapper.synchronizationAdd(kvt);

            // 根据 库名查询所有的表名
            String sql1 = "SELECT table_name FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA ='" + kvt.getDatabaseName() + "'";
            ResultSet resultSet = statement.executeQuery(sql1);
            //处理数据库查询结果的循环结构
            while (resultSet.next()) {
                //获取表名
                String tableName = resultSet.getString("table_name");

                //添加资产展示数据
                this.dataDisplayAdd(tableName,connection,kvt);
                //添加数据结构
                this.dataStructureAdd(connection,kvt,tableName);

            }

           this.selectDatabaseName(connection, kvt);


            Result<SysUser> sysUserResult = remoteUserSer.selectUserId(SecurityUtils.getUserId());
            SysUser data = sysUserResult.getData();
            DeptUserKvt build = DeptUserKvt.builder()
                    .userId(SecurityUtils.getUserId())
                    .kvtId(kvt.getId())
                    .deptId(data.getDeptId()).build();
            baseMapper.DeptUserKvtAdd(build);

            resultSet.close();
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return Result.success("mysql同步成功");
    }

    /**
     * 同步添加资产展示
     * @param tableName
     * @param connection
     * @param kvt
     */
    public void dataDisplayAdd(String tableName,Connection  connection,Kvt kvt) {
        try {
            //查询表中的数据
            String selectSql = "SELECT * FROM " + tableName;

            Statement stmt = connection.createStatement();

            //执行查询sql语句
            ResultSet resultSet2 = stmt.executeQuery(selectSql);

            ResultSetMetaData metaDataTest = resultSet2.getMetaData();

            int columnCount = metaDataTest.getColumnCount();


            List<DataDisplay> dataDisplays = new ArrayList<>();

            while (resultSet2.next()) {
                for (int i = 1; i <= columnCount; i++) {
                    DataDisplay dataDisplay = new DataDisplay();
                    String columnName = metaDataTest.getColumnName(i);
                    //资产展示类型
                    String typeName = metaDataTest.getColumnTypeName(i);
                    dataDisplay.setValue(resultSet2.getObject(i));
                    dataDisplay.setType(typeName);
                    dataDisplay.setWarName(kvt.getDatabaseName() + "-" + tableName + "-" + columnName);
                    dataDisplays.add(dataDisplay);
                }
            }

            System.out.println(dataDisplays);
            if(dataDisplays!=null  && dataDisplays.size()!=0){
                baseMapper.dataDisplayAdd(dataDisplays);
            }
            resultSet2.close();
        } catch (SQLException e) {
            throw new RuntimeException("添加资产展示数据有误",e);
        }
        log.info("资产展示添加成功");

    }


    /**
     *添加数据结构
     * @param connection
     * @param kvt
     * @param tableName
     */
    private void dataStructureAdd(Connection connection,Kvt kvt,String tableName){
        try{
            //执行一个预编译的 SQL 查询
            PreparedStatement preparedStatement = connection.prepareStatement(this.sql()
                    + "    TABLE_SCHEMA = ?\n"
                    + "    AND TABLE_NAME = ?");
            preparedStatement.setString(1, kvt.getDatabaseName()); // 设置第一个参数的值
            preparedStatement.setString(2, tableName); // 设置第二个参数的值
            ResultSet executeQuery = preparedStatement.executeQuery();
            while (executeQuery.next()) {

                String name = executeQuery.getString("name");
                /**
                 * 注释
                 */
                String comment = executeQuery.getString("comment");
                /**
                 * 是否主键
                 */
                String isPrimaryKey = executeQuery.getString("isPrimaryKey");
                /**
                 * 类型
                 */
                String type = executeQuery.getString("type");
                /**
                 * 映射类型
                 */
                String mappingType = executeQuery.getString("mappingType");
                /**
                 * 长度
                 */
                String length = executeQuery.getString("length");
                /**
                 * 小数位
                 */
                String decimalPlaces = executeQuery.getString("decimalPlaces");
                /**
                 * 是否为空
                 */
                String isNull = executeQuery.getString("isNull");
                /**
                 * 默认值
                 */
                String defaultValue = executeQuery.getString("defaultValue");

                DataStructure build = DataStructure.builder()
                        .tableName(tableName)
                        .warName(kvt.getDatabaseName())
                        .name(name)
                        .comment(comment)
                        .isPrimaryKey(isPrimaryKey)
                        .type(type)
                        .mappingType(mappingType)
                        .length(length)
                        .decimalPlaces(decimalPlaces)
                        .isNull(isNull)
                        .defaultValue(defaultValue)
                        .isDict(null)
                        .dictKey(null)
                        .build();
                baseMapper.add(build);
            }
            executeQuery.close();
        } catch (SQLException e) {
            throw new RuntimeException("添加数据结构失败");
        }
        log.info("添加数据结构成功");

    }

    /**
     * 查询数据库中表的总条数 以及备注 名称
     * @param connection
     * @param kvt
     */
    public void selectDatabaseName(Connection connection,Kvt kvt){
        try {
            //查询总条数
            String sql2 = "SELECT\n" +
                    "            TABLE_NAME as 'name',\n" +
                    "                TABLE_COMMENT as 'as',\n" +
                    "                TABLE_ROWS as 'dataTotal'\n" +
                    "        FROM INFORMATION_SCHEMA.TABLES\n" +
                    "        WHERE\n" +
                    "            TABLE_SCHEMA = ?";
            PreparedStatement preparedStatement2 = connection.prepareStatement(sql2);
            preparedStatement2.setString(1, kvt.getDatabaseName()); // 设置第一个参数的值
            ResultSet query = preparedStatement2.executeQuery();
            while (query.next()) {
                //表名
                String name = query.getString("name");
                //备注
                String as = query.getString("as");
                //总数量
                String dataTotal = query.getString("dataTotal");

                Child build1 = Child.builder()
                        .dataTotal(Integer.valueOf(dataTotal))
                        .name(name)
                        .as(as)
                        .isPrimaryKey(null)
                        .build();
                baseMapper.childAdd(build1);
            }

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        log.info("查询总条数成功");
    }

    public Result postgres(Kvt kvt) {
        Connection connection = null;
        Statement stmt = null;
        try {
            // 加载 PostgreSQL 驱动类
            Class.forName("org.postgresql.Driver");

            // 创建数据库连接
            String url = "jdbc:postgresql://" + kvt.getHost() + ":" + kvt.getPort() + "/" + kvt.getDatabaseName();
            String user = kvt.getUsername();
            String password = kvt.getPassword();
            connection = DriverManager.getConnection(url, user, password);

            Statement statement = connection.createStatement();
            ResultSet resultSet = stmt.executeQuery("SELECT table_name FROM information_schema.tables WHERE table_schema = 'public';");

            while (resultSet.next()) {
                String name = resultSet.getString("name");
                System.out.println("name" + name);
            }
            resultSet.close();
            statement.close();
            connection.close();
        } catch (ClassNotFoundException | SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                if (connection != null) {
                    connection.close();
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
        return Result.success("postgres同步成功");
    }

    /**
     * 同步数据
     *
     * @param kvt
     */
    @Override
    public Result synchronizationAdd(Kvt kvt) {
        //先判断哪种数据库
        if (kvt.getType() == 1) {
            mysql(kvt);
        } else {
            postgres(kvt);
        }
        return Result.success("同步成功");
    }

    @Override
    public void dictionaryTypeAdd(Dictionaryy diction) {
        baseMapper.dictionaryTypeAdd(diction);
    }

    @Override
    public void dictionTypeAdd(DictionaryType dictionaryType) {
        baseMapper.dictionTypeAdd(dictionaryType);
    }

    @Override
    public Dictionaryy selectDictionaryTypeId(Integer id) {
        return baseMapper.selectDictionaryTypeId(id);
    }

    @Override
    public void delDictionaryTypeId(Dictionaryy dictionaryy) {
        baseMapper.updateDictionaryTypeId(dictionaryy);
    }

    @Override
    public void utilAssetModel(DataStructure dataStructure) {
        baseMapper.utilAssetModel(dataStructure);
    }

    @Override
    public Result<OverallDataStructure> OverallDataStructure() {
        List<Synchronization> synchronizationList = baseMapper.selectSyn();
        List<Child> list = baseMapper.list();
        IntStream intStream = list.stream().mapToInt(Child::getDataTotal);
        OverallDataStructure build = OverallDataStructure.builder()
                .table(list.size())
                .datas(intStream.sum() + list.size())
                .kvt(synchronizationList.size())
                .build();

        return Result.success(build);
    }

    @Override
    public List<DataDisplay> listResult() {
        return baseMapper.listResult();
    }

    /**
     * 查询 根据库名跟表名查询字段信息
     * @return
     */
    private String sql () {
        //查询 根据库名跟表名查询字段信息
       return  "SELECT\n" +
                "            TABLE_NAME AS 'tableName',\n" +
                "                COLUMN_NAME AS 'name',\n" +
                "                COLUMN_COMMENT AS 'comment',\n" +
                "                CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END AS 'isPrimaryKey',\n" +
                "\n" +
                "                CASE\n" +
                "                    WHEN DATA_TYPE = 'int' THEN 'Integer'\n" +
                "                    WHEN DATA_TYPE = 'varchar' THEN 'String'\n" +
                "                    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal'\n" +
                "                    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean'\n" +
                "\n" +
                "                    ELSE DATA_TYPE\n" +
                "                    END AS 'type',\n" +
                "                DATA_TYPE AS 'mappingType',\n" +
                "                COLUMN_TYPE AS 'detailType',\n" +
                "                CHARACTER_MAXIMUM_LENGTH AS 'length',\n" +
                "                NUMERIC_SCALE AS 'decimalPlaces',\n" +
                "                IS_NULLABLE AS 'isNull',\n" +
                "                COLUMN_DEFAULT AS 'defaultValue'\n" +
                "        FROM\n" +
                "            INFORMATION_SCHEMA.COLUMNS\n" +
                "        WHERE\n";
    }
}
